(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/forms'), require('codemirror'), require('@angular/common')) :
    typeof define === 'function' && define.amd ? define('lt-codemirror', ['exports', '@angular/core', '@angular/forms', 'codemirror', '@angular/common'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['lt-codemirror'] = {}, global.ng.core, global.ng.forms, global.codemirror, global.ng.common));
}(this, (function (exports, core, forms, CodeMirror, common) { 'use strict';

    /**
     * CodeMirror component
     * Usage :
     * <lt-codemirror [(ngModel)]="data" [config]="{...}"></lt-codemirror>
     */
    var LtCodemirrorComponent = /** @class */ (function () {
        /**
         * Constructor
         */
        function LtCodemirrorComponent() {
            this.change = new core.EventEmitter();
            this.focus = new core.EventEmitter();
            this.blur = new core.EventEmitter();
            this.cursorActivity = new core.EventEmitter();
            this.instance = null;
            this.val = '';
        }
        Object.defineProperty(LtCodemirrorComponent.prototype, "value", {
            get: function () { return this.val; },
            set: function (v) {
                if (v !== this.val) {
                    this.val = v;
                    this.onChange(v);
                }
            },
            enumerable: false,
            configurable: true
        });
        /**
         * On component destroy
         */
        LtCodemirrorComponent.prototype.ngOnDestroy = function () {
        };
        /**
         * On component view init
         */
        LtCodemirrorComponent.prototype.ngAfterViewInit = function () {
            this.config = this.config || {};
            this.codemirrorInit(this.config, this.size);
        };
        /**
         * Initialize codemirror
         */
        LtCodemirrorComponent.prototype.codemirrorInit = function (config, size) {
            var _this = this;
            this.instance = CodeMirror.fromTextArea(this.host.nativeElement, config);
            this.instance.setValue(this.val);
            this.instance.on('change', function () {
                _this.updateValue(_this.instance.getValue());
            });
            this.instance.on('focus', function (instance, event) {
                _this.focus.emit({ instance: instance, event: event });
            });
            this.instance.on('cursorActivity', function (instance) {
                _this.cursorActivity.emit({ instance: instance });
            });
            this.instance.on('blur', function (instance, event) {
                _this.blur.emit({ instance: instance, event: event });
            });
            if (this.size != undefined) {
                this.instance.setSize(this.size.w, this.size.h);
            }
        };
        /**
         * Value update process
         */
        LtCodemirrorComponent.prototype.updateValue = function (value) {
            this.value = value;
            this.onTouched();
            this.change.emit(value);
        };
        /**
         * Implements ControlValueAccessor
         */
        LtCodemirrorComponent.prototype.writeValue = function (value) {
            this.val = value || '';
            if (this.instance) {
                this.instance.setValue(this.val);
            }
        };
        LtCodemirrorComponent.prototype.onChange = function (_) { };
        LtCodemirrorComponent.prototype.onTouched = function () { };
        LtCodemirrorComponent.prototype.registerOnChange = function (fn) { this.onChange = fn; };
        LtCodemirrorComponent.prototype.registerOnTouched = function (fn) { this.onTouched = fn; };
        return LtCodemirrorComponent;
    }());
    LtCodemirrorComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'lt-codemirror',
                    providers: [
                        {
                            provide: forms.NG_VALUE_ACCESSOR,
                            useExisting: core.forwardRef(function () { return LtCodemirrorComponent; }),
                            multi: true
                        }
                    ],
                    template: "\n    <textarea #host></textarea>\n"
                },] }
    ];
    LtCodemirrorComponent.ctorParameters = function () { return []; };
    LtCodemirrorComponent.propDecorators = {
        config: [{ type: core.Input }],
        size: [{ type: core.Input }],
        change: [{ type: core.Output }],
        focus: [{ type: core.Output }],
        blur: [{ type: core.Output }],
        cursorActivity: [{ type: core.Output }],
        host: [{ type: core.ViewChild, args: ['host', { static: false },] }],
        instance: [{ type: core.Output }],
        value: [{ type: core.Input }]
    };

    var LtCodemirrorModule = /** @class */ (function () {
        function LtCodemirrorModule() {
        }
        return LtCodemirrorModule;
    }());
    LtCodemirrorModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [
                        common.CommonModule
                    ],
                    declarations: [LtCodemirrorComponent],
                    exports: [LtCodemirrorComponent]
                },] }
    ];

    /*
     * Public API Surface of lt-codemirror
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.LtCodemirrorComponent = LtCodemirrorComponent;
    exports.LtCodemirrorModule = LtCodemirrorModule;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=lt-codemirror.umd.js.map
